/* soft link routines */

#include <mintbind.h>
#include <errno.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>

#ifdef __TURBOC__
# include <sys\stat.h>
#else
# include <sys/stat.h>
#endif

#include "lib.h"

/*
 * If MiNT 0.9 or later is active, use the kernel routines for these;
 * otherwise, try to choose failure modes that applications will best be
 * able to handle
 */

int
__symlink(old, new)
	const char *old, *new;
{
	char linknamebuf[PATH_MAX];
	char pathbuf[PATH_MAX];
	char* linkname = (char*) old;
	char* path = (char*) new;
	long r;

	if (!__libc_unix_names)
	  {
	    linkname = linknamebuf;
	    path = pathbuf;
	    _unx2dos(old, linkname, sizeof (linknamebuf));
	    _unx2dos(new, path, sizeof (pathbuf));
	  }
	r = Fsymlink(linkname, path);
	if (r) {
		struct stat sb;

		if ((r == -EPATH)) {
			if (_enoent(path))
				r = -ENOENT;
		} else if ((r == -EACCESS) && (!Fxattr(1, path, &sb)))
			r = -EEXIST;
		errno = (int) -r;
		return -1;
	}
	return (int) r;
}

#ifdef HAVE_WEAK_SYMBOLS
weak_alias (__symlink, symlink)
#endif
